/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 producer-consumer for MYO shared programming model
 This is example to implement multi-producer-consumer example with the shared 
 ringbuffer and myoSem and myoMutex.
 The ringbuffer(*buffer) is protected by mutex and sem. for example, one consumer 
 need to acquire the sem and mutex before reading the ringbuffer.
*/
#include "arena.h"

extern MyoError myoiUserInit(void);

void init() {
    *result = 0;
    myoArenaRelease(*resultarena);
    *bufferhead = * bufferrear = 0;
    *done = 0;
    myoMutexCreate(mutex);
    myoMutexCreate(resultmutex);
    myoSemCreate(0, productsem);
    myoSemCreate(MAXARRAY, spacesem);
}

void fini() {
     myoArenaDestroy(*resultarena);
     myoMutexDestroy(*mutex);
     myoMutexDestroy(*resultmutex);
     myoSemDestroy(*productsem);
     myoSemDestroy(*spacesem);
}

void producer() {
    int i, product;
    printf("I am the producer, I will generate numbers into buffer array\n");
    fflush(stdout);

    //produce 50 data
    for(i =0; i<50; i++) {
        myoSemWait(*spacesem);
        myoAcquire(); //acquire *bufferrear
        //generate data
        product = i;
        printf("produce the data:%d\n", product);
        fflush(stdout);

        //myoMutexLock(*mutex); //not needed, if only 1 producer
        buffer[*bufferhead] = product;
        *bufferhead = (*bufferhead +1) % MAXARRAY;
        myoRelease(); //release *bufferhead
        myoSemPost(*productsem);
        //myoMutexUnlock(*mutex); //not needed, if only 1 producer
    }
}

int main(int argc, char * argv[]){

    MyoiRFuncCallHandle iRPCHandler[MAXCONSUMER];
    int i;
	    
    if(MYO_SUCCESS != myoiLibInit(NULL, (void*)&myoiUserInit)) {
        printf("Failed to initialize MYO runtime\n");
        return -1;
    }

    printf("====================CPU Client==================\n");
    fflush(stdout);
    
    init();

    myoRelease(); //release before RPC

    for(i = 0 ; i< MAXCONSUMER; i++ ) {
        iRPCHandler[i] = myoiRemoteCall("consumer", (void*)i, 0);
    }

    producer();

    *done = 1;
    myoRelease(); //release *done

    for(i = 0 ; i< MAXCONSUMER; i++ ) {
        myoiGetResult(iRPCHandler[i]);
    }

    //myoAcquire(); //acquire after RPC return

    myoArenaAcquire(*resultarena);
    printf("result:%d\n", *result);
    fini();

    myoiLibFini();
    printf("================================================\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
   MyoError ret;
   ret = MYO_SUCCESS;
   if((resultarena = (MyoArena *)myoSharedMalloc(sizeof(MyoArena) )) == NULL) {
        fprintf(stderr, "myoSharedMalloc resultarena Failed\n");
        exit(1);
   }
   if(MYO_SUCCESS != myoArenaCreate(MYO_ARENA_OURS, 0, resultarena)) {
        printf("myoAllcoateArena Failed!\n");
        exit(1);
   }

   if((buffer = (int *)myoSharedMalloc(sizeof(int) * MAXARRAY)) == NULL) {
        fprintf(stderr, "myoSharedMalloc buffer Failed\n");
        exit(1);
   }
   if((bufferhead = (int*)myoSharedMalloc(sizeof(int) )) == NULL) {
        fprintf(stderr, "myoSharedMalloc bufferhead Failed NULL\n");
        exit(1);
   }
   if((bufferrear = (int*)myoSharedMalloc(sizeof(int) )) == NULL) {
        fprintf(stderr, "myoSharedMalloc bufferrear Failed NULL\n");
        exit(1);
   }
   if((result = (int  *)myoArenaMalloc(*resultarena, sizeof(int))) == NULL) {
        fprintf(stderr, "myoArenaMalloc result Failed\n");
        exit(1);
   }
   if((done = (int  *)myoSharedMalloc(sizeof(int))) == NULL) {
        fprintf(stderr, "myoSharedMalloc done Failed\n");
        exit(1);
   }
   if((mutex = (shared MyoMutex *)myoSharedMalloc(sizeof(shared MyoMutex))) ==  NULL) {
        fprintf(stderr, "myoSharedMalloc mutex Failed\n");
        exit(1);
   }
   if((resultmutex = (shared MyoMutex *)myoSharedMalloc(sizeof(shared MyoMutex ))) == NULL) {
        fprintf(stderr, "myoSharedMalloc resultmutex Failed\n");
        exit(1);
   }
   productsem = (shared MyoSem *)myoSharedMalloc(sizeof(shared MyoSem));
   if (!productsem) {
        printf("myoiUserInit: myoSharedMalloc productsem Failed!\n");
        exit(1);
   }
   spacesem = (shared MyoSem *)myoSharedMalloc(sizeof(shared MyoSem));
   if (!spacesem) {
        printf("myoiUserInit: myoSharedMalloc spacesem Failed!\n");
        exit(1);
   }

   myoiVarRegister((void *)&buffer, (char*)"buffer");
   myoiVarRegister((void *)&bufferhead, (char*)"bufferhead");
   myoiVarRegister((void *)&bufferrear, (char*)"bufferrear");
   myoiVarRegister((void *)&result, (char*)"result");
   myoiVarRegister((void *)&done, (char*)"done");
   myoiVarRegister((void *)&mutex, (char*)"mutex");
   myoiVarRegister((void *)&resultmutex, (char*)"resultmutex");
   myoiVarRegister((void *)&productsem, (char*)"productsem");
   myoiVarRegister((void *)&spacesem, (char*)"spacesem");
   myoiVarRegister((void *)&resultarena, (char*)"resultarena");

   return ret;
}
