/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 producer-consumer for MYO shared programming model
 This is example to implement multi-producer-consumer example with the shared 
 ringbuffer and myoSem and myoMutex.
 The ringbuffer(*buffer) is protected by mutex and sem. for example, one consumer 
 need to acquire the sem and mutex before reading the ringbuffer.
*/

#include "arena.h"

extern MyoError myoiUserInit(void);

//Remote function
void consumer(int threadid) {
    int product;
    printf("The consumer [%d], will consume the buffer array\n", threadid);
    fflush(stdout);

    myoAcquire();//acquire in the service

    while(1) {
        if(MYO_SUCCESS != myoSemTryWait(*productsem))  { //buffer empty;
            if(*done) { //producer stoped
                break; //producer stoped and buffer empty.
            } else {
                myoAcquire(); //acquire *done, wait for producing
                continue;
            }
            usleep(500); //sleep for a while, since no product
        }

        //retrieve the data
        myoMutexLock(*mutex);
        myoAcquire(); //acquire *bufferrear
        product = buffer[*bufferrear];
        *bufferrear = (*bufferrear+1) %MAXARRAY;
        myoRelease(); //release *bufferrear
        myoMutexUnlock(*mutex);
        myoSemPost(*spacesem); //post spacesem
        //consume data
        printf("[%d] consume the data:%d\n", threadid, product);
        fflush(stdout);
        myoMutexLock(*resultmutex); 
        myoArenaAcquire(*resultarena); //acquire resultarena
        *result += product;
        myoArenaRelease(*resultarena); //release *result
        myoMutexUnlock(*resultmutex);
    }
    printf("[%d] done\n", threadid);
    fflush(stdout);

    //myoRelease(); //release before return
}

int main(int argc, char * argv[]){
    printf("====================CARD Service==================\n");
    myoiLibInit(NULL, (void*)&myoiUserInit);
    myoiLibFini();
    printf("=================================================\n");
    return (0);
}


//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
    MyoError ret;
    ret = MYO_SUCCESS;

    ret = myoiRemoteFuncRegister((MyoiRemoteFuncType) consumer, "consumer");

    myoiVarRegister((void *)&buffer, "buffer");
    myoiVarRegister((void *)&bufferhead, "bufferhead");
    myoiVarRegister((void *)&bufferrear, "bufferrear");
    myoiVarRegister((void *)&result, "result");
    myoiVarRegister((void *)&done, "done");
    myoiVarRegister((void *)&mutex, "mutex");
    myoiVarRegister((void *)&resultmutex, "resultmutex");
    myoiVarRegister((void *)&productsem, "productsem");
    myoiVarRegister((void *)&spacesem, "spacesem");
    myoiVarRegister((void *)&resultarena, "resultarena");

    return ret;
}

