/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 One way arena for MYO shared programming model
 In this example, one global shared data(*array) is declared in HOST2DEVICE 
 arena. Host side random generates integer in it. One-way means card can access
 the array without the need to release the data back. Also runtime don't need
 the overhead of pagefault to record the dirty pages.
*/

#include "arrayop.h"
 
extern MyoError myoiUserInit(void);

unsigned long long  starttime, stoptime, executiontime;

shared ARRAYTYPE * array;
shared ARRAYTYPE * meanvalue;
#ifdef ONEWAYARENA
MyoArena arena;
#endif

#define RANDOM() random()

void initArray(ArrayArg * arrayarg)
{
    int i ;
    for(i=0; i<MAXARRAY; i++) {
        array[i] = (ARRAYTYPE) RANDOM() % MAXARRAY;
    }
    arrayarg->array=array;
#ifdef ONEWAYARENA
    arrayarg->arena=arena;
#endif
    arrayarg->length = MAXARRAY;
}

void printArray(ArrayArg * ar)
{
    int i;
    ARRAYTYPE * arr = ar->array;
    printf("\n--------------------\n");
    for(i=0; i<ar->length; i++) {
        printf("%d ", arr[i]);
    }
    printf("\n--------------------\n");
    fflush(stdout);
}

int main(int argc, char * argv[]){
    int iteration = 1;

    if(argc > 1) {// Usage: cmd <numRuns>
        iteration = atoi(argv[1]);
    }
	    
    MyoiRFuncCallHandle  rpcHandle ;
    if(MYO_SUCCESS != myoiLibInit(NULL, (void*)&myoiUserInit)) {
        printf("Failed to initialize MYO\n");
        exit(-1);
    }
    printf("====================CPU Client==================\n");
    fflush(stdout);
    ArrayArg * arg;
   if((arg = (ArrayArg *)myoSharedMalloc(sizeof(ArrayArg) )) == NULL) {
        fprintf(stderr, "myoSharedMalloc ArrayArg Failed\n");
        exit(1);
    }

    executiontime = 0; 
    while( (iteration--) !=0) {
        initArray(arg); //initialize the array
        myoRelease(); //release before RPC
#ifdef ONEWAYARENA
        myoArenaRelease(arena); //release before RPC
#endif
        //printArray(arg);

        starttime = myoWallTime();

        rpcHandle = myoiRemoteCall((char*)"calculatemean", arg, 0);
        myoiGetResult(rpcHandle);
        myoAcquire();//acquire after RPC return
        stoptime  = myoWallTime();
	executiontime += stoptime - starttime;

        printf("%d\n", *meanvalue);
        fflush(stdout);
    }
    printf("================================================\n");
    myoiLibFini();

    printf("execution time:%lld\n", executiontime);
    printf("*** Succeed\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
   MyoError ret;
   ret = MYO_SUCCESS;

#ifdef ONEWAYARENA
   //default OUR type
   //myoArenaCreate(MYO_ARENA_OURS, 0 , &arena);
   //ONE_WAY arena
   myoArenaCreate(MYO_ARENA_OURS, MYO_HOST_TO_DEVICE, &arena);
   if((array = (int *)myoArenaMalloc(arena, sizeof(int) * MAXARRAY )) == NULL) {
#else
   if((array = (int *)myoSharedMalloc(sizeof(int) * MAXARRAY )) == NULL) {
#endif
        fprintf(stderr, "myoSharedMalloc array Failed\n");
        exit(1);
    }
   myoiVarRegister((void *)&array, "array");

   if((meanvalue = (int *)myoSharedMalloc(sizeof(int))) == NULL) {
        fprintf(stderr, "myoSharedMalloc meanvalue Failed\n");
        exit(1);
    }
   myoiVarRegister((void *)&meanvalue, "meanvalue");

   return ret;
}
