/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 One way arena for MYO shared programming model
 In this example, one global shared data(*array) is declared in HOST2DEVICE 
 arena. Host side random generates integer in it. One-way means card can access
 the array without the need to release the data back. Also runtime don't need
 the overhead of pagefault to record the dirty pages.
*/

#include "arrayop.h"


extern MyoError myoiUserInit(void);

shared ARRAYTYPE * array;
shared ARRAYTYPE * meanvalue;

static int cmp(const void* a1p, const void* a2p)
{
    const ARRAYTYPE *a1 = (const ARRAYTYPE*)a1p;
    const ARRAYTYPE *a2 = (const ARRAYTYPE*)a2p;
    return (*a1 <= *a2)? 1: 0;
}

//Remote function
void calculatemean(ArrayArg * arg)
{
    myoAcquire(); //acquire in service
#ifdef ONEWAYARENA
    myoArenaAcquire(arg->arena);
#endif
    
    //printf("array:%p, length:%d\n", arg->array, arg->length);
    qsort(arg->array, arg->length, sizeof(ARRAYTYPE), cmp);
    * meanvalue = arg->array[arg->length/2];
    //printf("meanvalue:%d\n", *meanvalue);
    //fflush(stdout);
    
    myoRelease(); //release before return
}

int main(int argc, char * argv[]){
    printf("====================CARD Service==================\n");
    myoiLibInit(NULL, (void*)&myoiUserInit);
    myoiLibFini();
    printf("=================================================\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
    MyoError ret;
    ret = MYO_SUCCESS;

    ret = myoiRemoteFuncRegister((MyoiRemoteFuncType) calculatemean, "calculatemean");

    myoiVarRegister((void *)&array, "array");
    myoiVarRegister((void *)&meanvalue, "meanvalue");

    return ret;
}
