/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 One simple implementation of producer-consumer for MYO shared programming model
 *buffer points to one shared buffer with 2 elements. Via asynchonous RPC,
 host and MIC access the corresponding element in the *buffer simultaneously.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "myo.h"
#include "myoimpl.h"

extern MyoError myoiUserInit(void);

#define MAXNUM 10
#define shared
shared int * buffer;
shared int * result;

/* producer generate the ith product
 */
int produce(int i) {
    return i;
}

/* producer
 */
void producer() {
    int i; 
    int bufferindex = 1;
    printf("I am the producer, I will write numbers into buffer\n");
    fflush(stdout);
    MyoiRFuncCallHandle  rpcHandle[2] = {NULL, NULL};

    //reset the result
    *result = 0;

    for(i=0; i<MAXNUM; i++) {
        bufferindex = (bufferindex +1) %2;
        //wait for consumer done and release the buffer[bufferindex].
        if(rpcHandle[bufferindex]) {
            //printf("wait rpc\n");
            myoiGetResult(rpcHandle[bufferindex]);
            rpcHandle[bufferindex] = NULL;
            myoAcquire(); //acquire after RPC return
        }
        buffer[bufferindex] = produce(i);
        printf("\nproduce:%d\n", i);
        myoRelease(); //release before RPC
        //invoke asynchronous RPC to consume the data
        rpcHandle[bufferindex] = myoiRemoteCall((char*)"consumer", (void*)bufferindex, 0);
    }

    //wait for all consumers done
    for (i = 0; i < 2; i++) {
        if(rpcHandle[i]) {
            myoiGetResult(rpcHandle[i]);
            //printf("wait rpc\n");
        }
    }

    myoAcquire();//acquire after RPC return

    //print out the result
    printf("\nresult:%d\n", *result);
    fflush(stdout);
}

int main(int argc, char * argv[]){

    MyoiRFuncCallHandle rpcHandle ;
    if(MYO_SUCCESS != myoiLibInit(NULL, (void*)&myoiUserInit)) {
        printf("Failed to initialize MYO runtime\n");
        return -1;
    }

    printf("====================CPU Client==================\n");
    fflush(stdout);
    
    producer();

    myoiLibFini();
    printf("================================================\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
   MyoError ret;
   ret = MYO_SUCCESS;

   if((buffer = (int *)myoSharedMalloc(sizeof(int) * 2)) == NULL) {
        fprintf(stderr, "myoSharedMalloc buffer Failed\n");
        exit(1);
   }
   if((result = (int*)myoSharedMalloc(sizeof(int) )) == NULL) {
        fprintf(stderr, "myoSharedMalloc result Failed NULL\n");
        exit(1);
   }

   myoiVarRegister((void *)&buffer, (char*)"buffer");
   myoiVarRegister((void *)&result, (char*)"result");

   return ret;
}
