/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 One simple implementation of producer-consumer for MYO shared programming model
 *buffer points to one shared buffer with 2 elements. Via asynchonous RPC,
 host and MIC access the corresponding element in the *buffer simultaneously.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "myo.h"
#include "myoimpl.h"

extern MyoError myoiUserInit(void);

#define MAXNUM 10
#define shared
shared int * buffer;
shared int * result;

//Remote function
void consumer(int bufferindex)
{
    myoAcquire(); //acquire in service
    *result += buffer[bufferindex];
    printf("I am the consumer, I will consume the data:%d\n", buffer[bufferindex]);
    fflush(stdout);
    myoRelease(); //release before return

}

int main(int argc, char * argv[]){
    printf("====================CARD Service==================\n");
    myoiLibInit(NULL, (void*)&myoiUserInit);
    myoiLibFini();
    printf("=================================================\n");
    return (0);
}


//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
    MyoError ret;
    ret = MYO_SUCCESS;

    ret = myoiRemoteFuncRegister((MyoiRemoteFuncType) consumer, "consumer");

    myoiVarRegister((void *)&buffer, "buffer");
    myoiVarRegister((void *)&result, "result");

    return ret;
}

