/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */

/**
 Description:
 Helloworld for MYO shared programming model
 In this example, one global shared data(*buffer) is declared, and inited 
 at myoiUserInit(). After runtime init(myoiLibInit), both host and card can
 have the buffer pointed to the same address.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "myo.h"
#include "myoimpl.h"
 
extern MyoError myoiUserInit(void);

#define MAXWORLD 64
#define shared

shared char *buffer;
shared int *sa;
shared int *sb;

MyoiHostSharedVarEntry sharedVarTable[3];
MyoiHostSharedFptrEntry sharedFptrTable[1];

void (*kernelThunk)();

shared void kernel()
{
    myoAcquire(); //acquire in service
    strcpy((char*) buffer, "Hello world!");
    *sa = 1111; *sb = 2222;
    printf("%s %d %d\n", buffer, *sa, *sb);
    fflush(stdout);
    myoRelease(); //release before return

    return;
}


int main(int argc, char * argv[]) {

    MyoiRFuncCallHandle  rpcHandle ;

    if(MYO_SUCCESS != myoiLibInit(NULL, (void*)&myoiUserInit)) {
        printf("Failed to initialize MYO\n");
        exit(-1);
    }

    sharedVarTable[0].varName = "buffer";
    sharedVarTable[0].size = MAXWORLD;
    sharedVarTable[0].ptrToLocalPtrToShared = &buffer;

    sharedVarTable[1].varName = "sa";
    sharedVarTable[1].size = sizeof(int);
    sharedVarTable[1].ptrToLocalPtrToShared = &sa;

    sharedVarTable[2].varName = "sb";
    sharedVarTable[2].size = sizeof(int);
    sharedVarTable[2].ptrToLocalPtrToShared = &sb;
    myoiHostSharedMallocTableRegister((void *) sharedVarTable, 3, 0);
    sharedFptrTable[0].funcName = "kernel";
    sharedFptrTable[0].funcAddr = (void*)&kernel;
    sharedFptrTable[0].localThunkAddr = &kernelThunk;

    myoiHostFptrTableRegister((void *) sharedFptrTable, 1, 0);

    printf("====================CPU Client==================\n");
    fflush(stdout);
    kernelThunk();
    myoRelease(); //release before RPC
    myoiRemoteThunkCall((void*)kernelThunk, NULL,0);
    myoAcquire();//acquire after RPC return
    printf("%s %d %d\n", buffer, *sa, *sb);
    fflush(stdout);
    printf("================================================\n");
    myoiLibFini();
    
    printf("*** Succeed\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
    MyoError ret;
    ret = MYO_SUCCESS;

    return ret;
}
