/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */


/**
 Description:
 Helloworld for MYO shared programming model
 In this example, one global shared data(*buffer) is declared, and inited 
 at myoiUserInit(). After runtime init(myoiLibInit), both host and card can
 have the buffer pointed to the same address.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "myo.h"
#include "myoimpl.h"

extern MyoError myoiUserInit(void);

void (*kernelThunk)();

#define shared
shared char * buffer;
shared int *sa, *sb;

MyoiMicSharedVarEntry sharedVarTable[3];
MyoiTargetSharedFptrEntry sharedFptrTable[1];

shared void kernel()
{
    myoAcquire(); //acquire in service
    printf("%s %d %d\n", buffer, *sa, *sb);
    fflush(stdout);

    char temp[] = "world hello";
    strcpy((char*) buffer, temp);
    *sa = 2222; *sb = 1111;
    myoRelease(); //release before return

    return;
}

void *kernel_wrapper(void *arg)
{
    kernel();
    return NULL;
}

//compiler generates the following function executed automatically before main
MyoError __attribute__((constructor)) target_myo_init()
{
    sharedVarTable[0].varName = "buffer";
    sharedVarTable[0].ptrToLocalPtrToShared = &buffer;
    sharedVarTable[1].varName = "sa";
    sharedVarTable[1].ptrToLocalPtrToShared = &sa;
    sharedVarTable[2].varName = "sb";
    sharedVarTable[2].ptrToLocalPtrToShared = &sb;
    myoiTargetSharedMallocTableRegister((void *) sharedVarTable, 3, 0);

    sharedFptrTable[0].funcName = "kernel";
    sharedFptrTable[0].funcAddr = &kernel;
    sharedFptrTable[0].wrapFuncAddr = &kernel_wrapper;
    sharedFptrTable[0].localThunkAddr = &kernelThunk;
    myoiTargetFptrTableRegister((void *) sharedFptrTable, 1, 0);

    return MYO_SUCCESS;
}

int main(int argc, char * argv[]){
    printf("====================CARD Service==================\n");
    myoiLibInit(NULL, (void*)&myoiUserInit);
    myoiLibFini();
    printf("=================================================\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
    return MYO_SUCCESS;
}
