/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 linkedlist for MYO shared programming model
 In this example, one pointer-contained structure store in MYO shared space. 
 Host and card can walk through the linkedlist w/o the need of marshalling.
*/

#include "linkedlist.h"
 
extern MyoError myoiUserInit(void);

/** @FUNC initLinkedList
 ** init linkedlist 
 ** @PARAM: number: the node number of linked list
 ** @RETURN:
 **    the pointer to linked list                                                                                           
 ***/
void *initLinkedList(int number)
{
    LinkedList *head, *temp,*node;
    int i;

    head = (LinkedList *)myoSharedMalloc(sizeof(LinkedList));
    if(head==NULL) {
        fprintf(stderr, "myoSharedMalloc head failed!\n");
        exit(1);
    }

    srand((unsigned int)time(NULL));
    printf("-------Init a Linked list -------\n");
    /*construct the linked list in shared address space*/
    temp=head;
    for( i= 0 ; i< number; i++ ){
        node = (LinkedListNode *)myoSharedMalloc(sizeof(LinkedListNode));
        if(node==NULL) {
            fprintf(stderr, "myoSharedMalloc node failed!\n");
            exit(1);
        }
        node->next=NULL;
        node->data= rand()%1000;
        temp->next = node;
        printf("Node[%d] is %d\n",i,node->data);
        temp =node;        
    }
    printf("--------------------------------\n");
    return head;
}

/** @FUNC destroyLinkedList
 ** destroy linkedlist 
 ** @PARAM: head: the head node of linked list
 ** @RETURN:
 ***/
void destroyLinkedList(LinkedList *head)
{
    LinkedList *temp, *temp1;
    int i;

    /*check the parameter*/
    assert(head);
    printf("HOST: Destroy Linked list.\n");
    temp = head->next;
    while(temp!=NULL){
        temp1 = temp;
        temp=temp->next;
        myoSharedFree(temp1);
    }
    myoSharedFree(head);
}

/** @FUNC printLinkedList
 **  print a linked list
 ** @PARAM head: the head node of linked list 
 ** @RETURN:
 **    the pointer to linked list array
 ***/
void printLinkedList(LinkedList *head)
{
    LinkedListNode *temp;
    int i=0;
    
    /*check the parameter*/
    assert(head);
    printf("-------Print Linked list -------\n");
    temp = head->next;
    while(temp!=NULL){
        printf("Node[%d] is %d\n",i++,temp->data);
        temp=temp->next;
    }
    printf("--------------------------------\n");

}


int main(int argc, char * argv[]){

    LinkedList *head;
    MyoiRFuncCallHandle rpcHandle ;
    int result;
    int listSize = LIST_NODE_NUMS;
    
    
    if(MYO_SUCCESS != myoiLibInit(NULL, (void*)&myoiUserInit)) {
        printf("Failed to initialize MYO runtime\n");
        return -1;
    }

    printf("====================CPU Client==================\n");
    fflush(stdout);
    //Initialize the linked list.
    head = (LinkedList*) initLinkedList(listSize);
    
    myoRelease(); //release before RPC
    // Call the card function
    printf("HOST: remote call[calculateInMIC]\n");
    rpcHandle = myoiRemoteCall((char*)"calculateInMIC",head,0);
    myoiGetResult(rpcHandle);
    
    // Print out the linked list in CPU side.
    myoAcquire(); //acquire after RPC return
    printLinkedList(head);
    
    destroyLinkedList(head);

    printf("================================================\n");
    myoiLibFini();
    
    printf("*** Succeed\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
   MyoError ret;
   ret = MYO_SUCCESS;

   return ret;
}
