/*
 * Copyright 2012 Intel Corporation.
 *
 * This file is subject to the Intel Sample Source Code License. A copy
 * of the Intel Sample Source Code License is included.
 */
/**
 Description:
 Reverse/Forward Acceleration for MYO shared programming model
 In this example, host and card access one shared data(*hostbuffer), which is 
 created at host and pass its pointer to card via the parameter of 
 myoiRemoteCall(Forward Acceleration). The card calls the function on the 
 host using myoiRemotecall(Reverse Acceleration)
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "myo.h"
#include "myoimpl.h"

extern MyoError myoiUserInit(void);

#define shared


void  hostkernel(int* hostbuffer)
{
    int temp;
    printf("Inside host Kernel \n");
    myoAcquire(); //acquire in service
    temp = *hostbuffer;
    printf("Received from card: %d \n", *hostbuffer);
    *hostbuffer = temp * temp;
    myoRelease(); //release before return
}



int main(int argc, char * argv[]){

    MyoiRFuncCallHandle rpcHandle ;
    if(MYO_SUCCESS != myoiLibInit(NULL, (void*)&myoiUserInit)) {
        printf("Failed to initialize MYO runtime\n");
        return -1;
    }
    printf("====================CPU Client==================\n");
    fflush(stdout);
    shared int* hostbuffer;

    if((hostbuffer = (int*)myoSharedMalloc(sizeof(int) )) == NULL) {
        fprintf(stderr, "myoSharedMalloc hostbuffer Failed\n");
        exit(1);
    }
    

    *hostbuffer = 10; 
    printf("Sent to card %d\n", *hostbuffer);
    fflush(stdout);
    myoRelease(); //Release before RPC
    rpcHandle = myoiRemoteCall((char*)"kernel", hostbuffer, 0);
    myoiGetResult(rpcHandle);
    myoAcquire();//acquire after RPC return
    printf(" Final Result: %d\n", *hostbuffer);
    fflush(stdout);
    printf("================================================\n");
    myoiLibFini();
    
    printf("*** Succeed\n");
    return (0);
}

//Init Function, Generated by Translator or by Hand
MyoError myoiUserInit(void){
   MyoError ret;
   ret = MYO_SUCCESS;


   ret = myoiRemoteFuncRegister((MyoiRemoteFuncType) hostkernel, "hostkernel");
   return ret;
}
